-- lua/gemini/api.lua

local M = {}

-- Store conversation history
local conversation_history = {}

local function get_api_key()
	-- Check for environment variable
	local api_key = os.getenv("GEMINI_API_KEY")
	if api_key then
		return api_key
	end

	-- Check for Neovim global variable
	api_key = vim.g.gemini_api_key
	if api_key then
		return api_key
	end

	return nil -- API key not found
end

local function make_request(prompt, context)
	local api_key = get_api_key()

	if not api_key then
		vim.notify(
			"Google AI API key not set. Set GEMINI_API_KEY environment variable or g.gemini_api_key in your init.vim or init.lua.",
			vim.log.levels.ERROR
		)
		return nil
	end

	local model = "gemini-2.0-flash"
	local contents = {}
	
	-- Add conversation history to the request
	for _, message in ipairs(conversation_history) do
		table.insert(contents, {
			parts = {{
				text = message.role .. ": " .. message.content
			}}
		})
	end

	-- Add the current prompt
	if context then
		table.insert(contents, {
			parts = {{
				text = "Context:\n" .. context .. "\n\nUser: " .. prompt
			}}
		})
	else
		table.insert(contents, {
			parts = {{
				text = "User: " .. prompt
			}}
		})
	end

	local payload = vim.json.encode({
		contents = contents,
	})

	-- Escape the payload for shell
	payload = vim.fn.shellescape(payload)

	local command = string.format(
		"curl -s -X POST "
			.. "'https://generativelanguage.googleapis.com/v1/models/%s:generateContent?key=%s' "
			.. "-H 'Content-Type: application/json' "
			.. "-d %s",
		model,
		api_key,
		payload
	)

	local result = vim.fn.system(command)

	-- Check for errors during the curl execution
	if vim.v.shell_error ~= 0 then
		vim.notify("Error executing curl. Check your command and ensure curl is installed.", vim.log.levels.ERROR)
		return nil
	end

	local success, decoded_result = pcall(vim.json.decode, result)
	if not success then
		vim.notify("Failed to decode API response: " .. result, vim.log.levels.ERROR)
		return nil
	end

	return decoded_result
end

function M.get_response(prompt, context)
	-- Add user message to history
	table.insert(conversation_history, {
		role = "User",
		content = prompt
	})

	local result = make_request(prompt, context)

	if result then
		if result.error then
			vim.notify("API Error: " .. vim.inspect(result.error), vim.log.levels.ERROR)
			return nil
		end

		if
			result.candidates
			and result.candidates[1]
			and result.candidates[1].content
			and result.candidates[1].content.parts
			and result.candidates[1].content.parts[1]
			and result.candidates[1].content.parts[1].text
		then
			local response_text = result.candidates[1].content.parts[1].text
			-- Add assistant response to history
			table.insert(conversation_history, {
				role = "Assistant",
				content = response_text
			})
			return response_text
		end
		
		vim.notify("Unexpected response structure: " .. vim.inspect(result), vim.log.levels.ERROR)
	end
	
	vim.notify("No response from Google AI API or malformed response.", vim.log.levels.ERROR)
	return nil
end

-- Add function to clear conversation history
function M.clear_conversation()
	conversation_history = {}
end

return M
